#include "clock-arch.h"
#include "target.h"
#include "irq.h"
#include "lpc24xx.h"

volatile clock_time_t Ticks;

/* Timer 0 interrupt handler */
static void Timer0Handler (void) __irq 
{ 
  T0IR = 1;			/* clear interrupt flag */
  IENABLE;			/* handles nested interrupt */

  ++Ticks;

  IDISABLE;
  VICVectAddr = 0;	/* Acknowledge Interrupt */
}

/* Timer init */
void clock_init(void)
{
  Ticks = 0;

  // Init Time0
  PCONP |= (1<<1); // Enable TMR0 clk
  T0TCR = 0x2;	// counting  disable and set reset
  T0TCR = 0x0; 	// release reset
  T0CTCR = 0;	// Timer Mode: every rising PCLK edge
  T0MCR = 0x3;	// Enable Interrupt on MR0, Enable reset on MR0

  // set timer 0 period
  T0PR = 0;
  T0MR0 = Fpclk / (CLOCK_CONF_SECOND);

  // init timer 0 interrupt
  T0IR |= (1<<0);	// clear pending interrupt
	install_irq( TIMER0_INT, (void *)Timer0Handler, 2 );
	
	T0TCR |= (1<<0);	// counting Enable 

}

/* returned The current clock time, measured in system ticks */
clock_time_t clock_time(void)
{
  return(Ticks);
}
