#include "emac.h"
#include <string.h>
#include <stdio.h>

void   rx_descr_init(void);
void   tx_descr_init(void);
void write_PHY (UNS_32 PhyReg, UNS_32 Value);
UNS_16 read_PHY (UNS_8 PhyReg) ;

/* Init the lpc23xx/24xx ethernet */
BOOL_8 tapdev_init(void)
{
  	UNS_32 regVal,to, tout;
	static volatile UNS_32 Tst;

  /* turn on the ethernet MAC clock in PCONP, bit 30 */
  regVal = PCONP;
  regVal |= PCONP_EMAC_CLOCK;
  PCONP = regVal;

#if RMII
  /* P1.6, ENET-TX_CLK, has to be set for EMAC to address a BUG in the engineering 
   version, even if this pin is not used for RMII interface. This bug has been fixed,
   and this port pin can be used as GPIO pin in the future release. */ 
  /* Unfortunately, this MCB2300 board still has the old eng. version LPC23xx chip
   on it. On the new rev.(xxAY, released on 06/22/2007), P1.6 should NOT be set.
  See errata for more details. */
  regVal = MAC_MODULEID;
  if ( regVal == OLD_EMAC_MODULE_ID ) 
  {
	/* This is the rev."-" ID ,
	on rev. A, regVal should NOT equal to OLD_EMAC_MODULE_ID,
	P1.6 should NOT be set. */
	PINSEL2 = 0x50151105;	/* selects P1[0,1,4,6,8,9,10,14,15] */
  }
  else
  {
	PINSEL2 = 0x50150105;	/* selects P1[0,1,4,8,9,10,14,15] */
  }
  PINSEL3 = 0x00000005;	/* selects P1[17:16] */
#else					/* else RMII, then it's MII mode */      
  PINSEL2 = 0x55555555;	/* selects P1[15:0] */
  PINSEL3 = 0x00000005;	/* selects P1[17:16] */
#endif


  // Reset entire MAC
  MAC_MAC1 = MAC1_RES_TX | MAC1_RES_MCS_TX | MAC1_RES_RX | MAC1_RES_MCS_RX |
             MAC1_SIM_RES | MAC1_SOFT_RES;	//MAC1 = 0x0000CF00;
	// reset all control registers		
  MAC_COMMAND = CR_REG_RES | CR_TX_RES | CR_RX_RES;	 //COMMAND = 0x0038;
  for (tout = 100; tout; tout--);  	/* A short delay after reset. */  
  MAC_MAC1 = 0; /* deassert all of the above soft resets in MAC1 */


  /* Enable Reduced MII interface. */
  MAC_COMMAND = CR_RMII | CR_PASS_RUNT_FRM | CR_PASS_RX_FILT;  
    /* Receive Broadcast and Perfect Match Packets */
  MAC_RXFILTERCTRL = RFC_BCAST_EN | RFC_PERFECT_EN;

  MAC_SUPP = 0;

  /* Set the Ethernet MAC Address registers */
  MAC_SA0 = (EMAC_ADDR1<<8) | EMAC_ADDR0;
  MAC_SA1 = (EMAC_ADDR3<<8) | EMAC_ADDR2;
  MAC_SA2 = (EMAC_ADDR5<<8) | EMAC_ADDR4;
 
  MAC_MAXF = ETH_MAX_FLEN;  //0x600;  

/*PCLK=12MHz, clock select=6, MDC=12/6=2MHz */  
  MAC_MCFG = 0x8018; 
  MAC_MCFG &= ~(1<<15);	/*release reset*/

  /* Initialize MAC control registers. */
  MAC_MAC1 = MAC1_PASS_ALL;
  MAC_MAC2 = MAC2_CRC_EN | MAC2_PAD_EN;
 


  MAC_CLRT = CLRT_DEF;
  MAC_IPGR = IPGR_DEF;
  MAC_IPGT = IPGT_HALF_DUP;

  write_PHY(PHY_BMCR,BMCR_RESET);
   
  to = PHY_TO;
  while(to)
  {
    regVal = read_PHY(PHY_BMCR);
    if(!(regVal & BMCR_RESET))
    {
      break;
    }
  }

  if(!to)
  {
    return(FALSE);
  }

  if(read_PHY(PHY_BMSR ) & BMSR_NOPREAM)
  {
    // PHY support preamble suppression
    MAC_MCFG |= 1 << 1;
  }

#if AUTO_NEGOTIATION_ENA != 0
  // PHY interrupt init, clear and enable
  // Set Auto-Negotiation Advertisement
  write_PHY(PhyAddr,PHY_ANAR, 1 | ANAR_10BT | ANAR_10BT_FULL | ANAR_100BT | ANAR_100BT_FULL);
  // Enable Auto-Negotiation
  regVal = BMCR_AN | BMCR_RE_AN;
#else

  regVal = 0;

  if(FIX_DUPLEX == FULL_DUPLEX)
  {
    regVal |= BMCR_DUPLEX;
  }

  if(FIX_SPEED == SPEED_100)
  {
    regVal |= BMCR_SPEED_100;
  }

#endif

  write_PHY(PHY_BMCR,regVal);

  to = PHY_TO; 
  while(to)
  {
    --to;
    if(read_PHY(PHY_BMSR) & BMSR_LINK_ESTABLISHED)
    {
      break;
    }
  }
  if(!to)
  {
    return(FALSE);
  }

  regVal = read_PHY(PHY_PHYSTS) & (PHYCR_LINK | PHYCR_SPEED | PHYCR_FULL_DUPEX);

  switch(regVal)
  {
  case 0x03:  // 10 BASE T Half-duplex
    printf("10 BASE T Half-duplex\r\n");
    regVal = 0;
    break;
  case 0x01:  // 100 BASE TX Half-duplex
    printf("100 BASE TX Half-duplex\r\n");
    regVal = BMCR_SPEED_100;
	MAC_SUPP |= (1<<8);
    break;
  case 0x07: // 10 BASE T Full-duplex
    printf("10 BASE T Full-duplex\r\n");
    regVal = BMCR_DUPLEX;
    MAC_IPGT = IPGT_FULL_DUP;
	MAC_MAC2 |= (1<<0);
	MAC_COMMAND |= 	(1<<10);
    break;
  case 0x5: // 100 BASE TX Full-duplex
    printf("100 BASE TX Full-duplex\r\n");
    regVal = BMCR_SPEED_100 | BMCR_DUPLEX;
    MAC_IPGT = IPGT_FULL_DUP;
	MAC_MAC2 |= (1<<0);
	MAC_COMMAND |= 	(1<<10);
	MAC_SUPP |= (1<<8);
    break;
  default:
    return(FALSE);
  }

  // Disable Auto-Negotiation and update speed and duplex settings
  write_PHY(PHY_BMCR,regVal);

  to = PHY_TO;
  while(to)
  {
    --to;
    if(read_PHY(PHY_BMSR) & BMSR_LINK_ESTABLISHED)
    {
      break;
    }
  }	 
  if(!to)
  {
    return(FALSE);
  }
 
  rx_descr_init();
  tx_descr_init();

  /* Enable receive and transmit mode of MAC Ethernet core */
  MAC_COMMAND  |= (CR_RX_EN | CR_TX_EN);
  MAC_MAC1     |= MAC1_REC_EN;

  return(TRUE);
}

/* Writes a value on the PHY registers */
static void write_PHY (UNS_32 PhyReg, UNS_32 Value)
{
  unsigned int tout;

  MAC_MADR = DP83848C_DEF_ADR | PhyReg;
  MAC_MWTD = Value;

  /* Wait utill operation completed */
  tout = 0;
  for (tout = 0; tout < MII_WR_TOUT; tout++) {
    if ((MAC_MIND & MIND_BUSY) == 0) {
      break;
    }
  }   
}


/* Read a value from the PHY registers */
static UNS_16 read_PHY (UNS_8 PhyReg)
{
  UNS_32 tout;

  MAC_MADR = DP83848C_DEF_ADR | PhyReg;
  MAC_MCMD = MCMD_READ;

  /* Wait until operation completed */
  tout = 0;
  for (tout = 0; tout < MII_RD_TOUT; tout++) {
    if ((MAC_MIND & MIND_BUSY) == 0) {
      break;
    }
  }
  MAC_MCMD = 0;
  return (MAC_MRDD);  
}


/* receive an Ethernet frame from MAC/DMA controller */
UNS_32 tapdev_read(void * pPacket)
{
	UNS_32 RxProduceIndex = MAC_RXPRODUCEINDEX;
	UNS_32 RxConsumeIndex = MAC_RXCONSUMEINDEX;
	UNS_32 Size = EMAC_MAX_PACKET_SIZE;
	UNS_32 in_size;

  if(RxConsumeIndex == RxProduceIndex)	// empty
  {
    return(0);
  }

  in_size = (RX_STAT_INFO(RxConsumeIndex) & 0x7ff)+1;  
  Size = MIN(Size,in_size);
  memcpy(pPacket,(unsigned int *)RX_BUF(RxConsumeIndex),Size);

  RxConsumeIndex++;
  if(RxConsumeIndex > MAC_RXDESCRIPTORNUM)
	RxConsumeIndex = 0;

  MAC_RXCONSUMEINDEX = RxConsumeIndex;
  return(Size);  
}

/* transmit an Ethernet frame to MAC/DMA controller */
BOOL_8 tapdev_send(void *pPacket, UNS_32 size)
{

	UNS_32 TxProduceIndex = MAC_TXPRODUCEINDEX;
	UNS_32 TxConsumeIndex = MAC_TXCONSUMEINDEX;
	UNS_32 TxProduceIndex_tmp = TxProduceIndex;	

  if(size == 0)
    return(TRUE);

  TxProduceIndex_tmp++;	
  if(TxProduceIndex_tmp > MAC_TXDESCRIPTORNUM)
    TxProduceIndex_tmp = 0;

  if(TxProduceIndex_tmp == TxConsumeIndex)  //full
    return(FALSE); 

  size = MIN(size,EMAC_MAX_PACKET_SIZE);
  memcpy((unsigned int *)TX_BUF(TxProduceIndex),pPacket,size);  
  TX_DESC_CTRL(TxProduceIndex) &= ~0x7ff;
  TX_DESC_CTRL(TxProduceIndex) |= (size - 1) & 0x7ff; 

  MAC_TXPRODUCEINDEX = TxProduceIndex_tmp;	 // update produce index, 
  										// and transmit now

  return(TRUE); 
}

// Keil: function added to initialize Rx Descriptors
static void rx_descr_init (void)
{
  UNS_32 i;

  for (i = 0; i < NUM_RX_FRAG; i++) {
    RX_DESC_PACKET(i)  = RX_BUF(i);
    RX_DESC_CTRL(i)    = (1<<31) | (1<<30) | (1<<29) | (1<<28) | (1<<26) | (EMAC_MAX_PACKET_SIZE-1);//RCTRL_INT | (ETH_FRAG_SIZE-1);
    RX_STAT_INFO(i)    = 0;
    RX_STAT_HASHCRC(i) = 0;
  }

  /* Set EMAC Receive Descriptor Registers. */
  MAC_RXDESCRIPTOR    = RX_DESC_BASE;
  MAC_RXSTATUS        = RX_STAT_BASE;
  MAC_RXDESCRIPTORNUM = NUM_RX_FRAG-1;

  /* Rx Descriptors Point to 0 */
  MAC_RXCONSUMEINDEX  = 0;
}


// Keil: function added to initialize Tx Descriptors
static void tx_descr_init (void) {
  UNS_32 i;

  for (i = 0; i < NUM_TX_FRAG; i++) {
    TX_DESC_PACKET(i) = TX_BUF(i);
    TX_DESC_CTRL(i)   = (1<<31) | (1<<30) | (1<<29) | (1<<28) | (1<<26) | (ETH_FRAG_SIZE-1);
    TX_STAT_INFO(i)   = 0;
  }

  /* Set EMAC Transmit Descriptor Registers. */
  MAC_TXDESCRIPTOR    = TX_DESC_BASE;
  MAC_TXSTATUS        = TX_STAT_BASE;
  MAC_TXDESCRIPTORNUM = NUM_TX_FRAG-1;

  /* Tx Descriptors Point to 0 */
  MAC_TXPRODUCEINDEX  = 0;
}

